@@ -4,7 +4,7 @@
 #
 # For something a little more involved, check out session_handler.pl
 
-package HTML::Mason;
+package MyMason::MyApp;
 
 # Bring in main Mason package.
 use HTML::Mason;
@@ -13,7 +13,6 @@ use HTML::Mason;
 # Uncomment the second line (and comment the first) to use
 # Apache::Request instead of CGI.pm to parse arguments.
 use HTML::Mason::ApacheHandler;
-# use HTML::Mason::ApacheHandler (args_method=>'mod_perl');
 
 # Uncomment the next line if you plan to use the Mason previewer.
 #use HTML::Mason::Preview;
@@ -26,20 +25,15 @@ use strict;
 #   use CGI;
 #}
 
-# Create Mason objects
+# Create ApacheHandler object
 #
-my $parser = new HTML::Mason::Parser;
-my $interp = new HTML::Mason::Interp (parser=>$parser,
-                                      comp_root=>'<component root>',
-                                      data_dir=>'<data directory>');
-my $ah = new HTML::Mason::ApacheHandler (interp=>$interp);
+my $ah = new HTML::Mason::ApacheHandler( comp_root => '<component root>',
+                                         data_dir => '<data directory>' );
 
 # Activate the following if running httpd as root (the normal case).
-# Resets ownership of all files created by Mason at startup. Change
-# these to match your server's 'User' and 'Group'.
+# Resets ownership of all files created by Mason at startup.
 #
-#chown (scalar(getpwnam "nobody"), scalar(getgrnam "nobody"),
-#       $interp->files_written);
+#chown (Apache->server->uid, Apache->server->gid, $ah->interp->files_written);
 
 sub handler
 {
