@@ -2,7 +2,7 @@
 
 package Bundle::HTML::Mason;
 
-$VERSION = '0.04';
+$VERSION = '0.05';
 
 1;
 
@@ -20,7 +20,15 @@ C<perl -MCPAN -e 'install Bundle::HTML::Mason'>
 
 Data::Dumper
 
-MLDBM
+File::Spec  0.8
+
+Params::Validate  0.13
+
+Apache::Request  0.31
+
+Exception::Class  1.00
+
+Cache::Cache  0.99
 
 HTML::Mason
 
